/*  micropower micrologger test-program to check the onboard TMP11x (112, 116)
    sensor and the EEPROM by reading temperatures and writing three blocks to the EEPROM

    See: https://hackaday.io/project/160269-micropower-micrologger-mpml for details
    like board files, circuit etc.

    Feel free to copy/share this file/code and change it however you like
*/

#include <extEEPROM.h>     // Credit to https://github.com/PaoloP74/extEEPROM and https://github.com/JChristensen/extEEPROM for their extEEPROM-code!
#include "LowPower.h"      // RocketScream low-power library. See https://github.com/rocketscream/Low-Power for details!

#define TMP_ADDR 0x49      // address of the onboard TMP11x
#define AUX_POWER_PIN A2   // for microPowerMicrologger = A2, power to all peripherals

//extEEPROM eep(kbits_512, 1, 64);         // 24LC512 = 512 Kibibit = 65536 Byte, 1x EEPROM, Page Size 64Byte (datasheet)
extEEPROM eep(kbits_2048, 1, 256);         // M24M02 = 2 Mibibit = 262144 Byte, 1x EEPROM, Page Size 256Byte

const uint8_t EE_firstAddress = 0;         // EEPROM starts at address zero
const long EE_lastAdress = 262143;         // 2Mbit = ((2048)Byte/8) = 256Bit * 1024 = 262144Byte
uint8_t logSize = sizeof(double);          // temperature comes as double (== float) from the sensor

// defining three positions for writing/reading. @start, @roughly in the middle, @end of eeprom
const uint8_t writeLogs = 100;
const long addr_firstBlockStart    = EE_firstAddress;
const long addr_middleBlockStart   = 262144 / 2;
const long addr_endBlockStart      = EE_lastAdress - (writeLogs * logSize);

double tempC      = 99.99;  // start-value = 99.99 so you can see if writing/reading does not work
double tempReadC  = 99.99;

// ---------------------------------------------------------------------------------------------------------------------------

void setup(void)
{
  pinMode(AUX_POWER_PIN, OUTPUT);     // fire up all peripherals. not needed if solder-jumper is set to "always on"
  digitalWrite(AUX_POWER_PIN, HIGH);
  delay(250); // let the voltage settle/charge the onboard caps

  TMP11xInit();

  Serial.begin(9600);

  long maxLogsEEPROM = (EE_lastAdress / logSize); // max. amount of logs in EEPROM
  Serial.print(F("Size of each temperature reading: ")); Serial.println(logSize);
  Serial.print(F("EEPROM holds ")); Serial.print(maxLogsEEPROM); Serial.print(F(" logs of ")); Serial.print(logSize); Serial.println(F(" bytes each."));

  uint8_t eepStatus = eep.begin(eep.twiClock400kHz);   // 400kHz, alternatively eep.twiClock100kHz

  if (eepStatus) {
    Serial.print(F("extEEPROM.begin() failed, status = "));
    Serial.println(eepStatus);
    while (1);
  }

  static long writeAddress  = EE_firstAddress;
  static long readAddress   = EE_firstAddress;

  // ----------------------------------------------------------------- Write values to EEPROM
  Serial.println(F(" "));
  Serial.println(F("SEND ANY CHARACTER to START writing three blocks of data"));
  Serial.println(F(""));

  while (Serial.available() == 0);  // wait for Serial input to start (any caharacter)

  long writeCounter = millis();
  writeXlogs(addr_firstBlockStart, 100);
  Serial.println(F("Block 1 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - writeCounter)); Serial.println(F(" milliseconds to write the values."));

  writeCounter = millis();
  writeXlogs(addr_middleBlockStart, 100);
  Serial.println(F("Block 2 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - writeCounter)); Serial.println(F(" milliseconds to write the values."));

  writeCounter = millis();
  writeXlogs(addr_endBlockStart, 100);
  Serial.println(F("Block 3 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - writeCounter)); Serial.println(F(" milliseconds to write the values."));

  Serial.println(F(""));
  delay(100);

  // ----------------------------------------------------------------- Read values from EEPROM
  Serial.println(F(" "));
  Serial.println(F("Starting to read out three blocks of data"));
  Serial.println(F(""));

  long readCounter = millis();
  readXlogs(addr_firstBlockStart, 100);
  Serial.println(F("Block 1 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - readCounter)); Serial.println(F(" milliseconds to read the values."));
  Serial.println(F(""));

  readCounter = millis();
  readXlogs(addr_middleBlockStart, 100);
  Serial.println(F("Block 2 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - readCounter)); Serial.println(F(" milliseconds to read the values."));
  Serial.println(F(""));

  readCounter = millis();
  readXlogs(addr_endBlockStart, 100);
  Serial.println(F("Block 3 - Finished"));
  Serial.print(F("It took "));  Serial.print((millis() - readCounter)); Serial.println(F(" milliseconds to read the values."));
  Serial.println(F(""));

  Serial.println(F("Shutdown..."));
  digitalWrite(AUX_POWER_PIN, LOW);
  delay(250);

  LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);

  // ---------------------------------------------------------------------------------------------------------------------------
}

void TMP11xInit() {
  Wire.beginTransmission(TMP_ADDR);

  Wire.write(0x01); // choose 8Bit config-register to write following setup-info to
  Wire.write(B01100000);   //config Byte 1 (OS  R1  R0 F1 F0 POL TM SD) -> see datasheet for options
  Wire.write(B10100000);   //config Byte 2 (CR1 CR0 AL EM 0  0   0  0)

  /* Set config byte 2 bit 7 and 6:
     CR1 = 0, CR0 = 0 = 0.25Hz conversion rate (once every 4 seconds)
     CR0 = 0, CR0 = 1 = 1.00Hz conversion rate (once every 1 second)
     CR0 = 1, CR0 = 0 = 4.00Hz conversion rate (4 times a second)
     CR0 = 1, CR0 = 1 = 8.00Hz conversion rate (8 times a second)
  */
  bool configStatus = Wire.endTransmission(); // gets return-value (error-check)
  Serial.print("Config written OK if value is 0: ");
  Serial.println(configStatus);
}

void TMP11xReadTempC() {
  Wire.beginTransmission(TMP_ADDR);
  Wire.write(0X00); // choose 8Bit temp-register to read temperature info
  Wire.endTransmission();

  Wire.requestFrom(TMP_ADDR, 2);  // Request 2 bytes , MSB first

  while (Wire.available())
  {
    int msb = Wire.read();
    int lsb = Wire.read();

    Wire.endTransmission();
    // Data conversion in raw values
    int rawtmp = msb << 8 | lsb;
    int value = rawtmp >> 4;
    tempC = value * 0.0625;
  }
}

void writeXlogs(long address, uint16_t ammountOfValues)
{
  static long writeAddress = address;
  for (uint16_t i = 0; i < ammountOfValues; i++)
  {
    TMP11xReadTempC();
    eep.write(writeAddress, (uint8_t*)(&tempC), logSize);
    writeAddress += logSize;
    Serial.print(i + 1); Serial.print(F(" - ")); Serial.println(tempC);
  }
}

void readXlogs(long address, uint16_t ammountOfValues)
{
  static long readAddress = address;
  for (uint16_t i = 0; i < ammountOfValues; i++)
  {
    eep.read(readAddress, (uint8_t*)(&tempC), logSize);
    readAddress += logSize;
    Serial.print(i + 1); Serial.print(F(" - ")); Serial.println(tempC);
  }
}


void loop(void)
{
  // nothing happening here
}
